; +
; !TF:	!ZF
;  TF:	0, ZF
;
; 86:	11 -> 00(TF)
; CM:	11 -> SL(TF)
;
; This function determines if the current execution mode is 86 (x86) or CM (Compatibility Mode).
; It does this by checking the value of the TF (Trace) flag, which is used for debugging.
; In 86 mode, TF is set to 11, while in CM mode, it is set to the value of the Stack Limit (SL).
;
; Returns:
;    0x11 if TF is set (86 mode)
;    -1 if TF is not set (CM mode)
;
TrRpl proc uses ebx
	mov ecx, 8   ; Loop counter
	xor ebx, ebx ; Initialize ebx to 0
	mov dx,  ss  ; Get the current SS register value
	.repeat
		mov eax, 11B ; Move the value 0x11B (the TF bit in the segment descriptor) into eax
		mov ss,  dx  ; Set SS to the current value (dx)
		mov ds,  ax  ; Move the value in ax into ds (which is 0x11B)
		mov ss,  dx  ; Set SS back to the current value (dx)
		mov ax,  ds  ; Move the value in ds (which is 0x11B) into ax
		mov ds,  dx  ; Set SS back to the current value (dx)
		add ebx, eax ; Add the value in eax (0x11B) to ebx
	.untilcxz          ; Repeat until ecx becomes 0
	mov       eax, ebx ; Move the final value in ebx into eax
	.if       Eax      ; If eax is not 0
		shr eax, 3  ; Shift eax right by 3 bits
		sub ax,  dx ; Subtract the value in dx from ax
	.endif
	ret ; Return to the caller
TrRpl  endp

; +
;  TF:	CF, 0x8
; !TF:	!CF
;
; This function gets the value of the GS segment register in 86 mode (x86). 
; It uses a technique where the GS register is temporarily set to the value of the SS register, 
; then read back to determine its value.
;
; Returns:
;    The value of the GS register.
;
; Notes:
;    This function relies on the fact that in 86 mode, the value of the GS register is stored in the lower 16 bits of the 32-bit GS register. 
;    Therefore, this function essentially reads the lower 16 bits of GS as a 32-bit value.
;
Tr86Gs proc C
	mov  ax, ss ; Move the current SS register value into ax
	mov  ss, ax ; Set SS to the value in ax
	mov  gs, ax ; Set GS to the value in ax
	mov  ss, ax ; Set SS back to the value in ax
	mov  ax, gs ; Move the value in gs (which is the value of SS) into ax
	test ax, ax ; Check if the value in ax is 0
	ret         ; Return to the caller
Tr86Gs endp

; +
;  TF:	CF, 0x8
; !TF:	!CF
;
; This function gets the value of the GS segment register in Compatibility Mode (CM).
; It utilizes the fact that in CM mode, the value of GS is stored in the upper 16 bits of the 32-bit GS register, 
; while the lower 16 bits are set to the value of the Stack Limit (SL).
; 
; This function simulates a specific scenario:
;    1. It temporarily sets the SS register to the value of GS.
;    2. It checks if the lower 16 bits of GS are equal to the value of the Stack Limit (SL).
;    3. If they are equal, it concludes that the mode is CM.
;    4. It then extracts the upper 16 bits of GS and returns them as the value.
;
; Returns:
;    The upper 16 bits of the GS register in Compatibility Mode (CM).
;
; Notes:
;    This function is specific to Compatibility Mode and may not work correctly in 86 mode.
;
TrCMGs proc uses ebx esi
	mov esi, 8   ; Loop counter
	xor eax, eax ; Initialize eax to 0
	xor ecx, ecx ; Initialize ecx to 0
	xor edx, edx ; Initialize edx to 0
	xor ebx, ebx ; Initialize ebx to 0
	mov cx,  ss  ; Get the current SS register value into cx
	.repeat
		mov ss,  cx  ; Set SS to the current value (cx)
		mov gs,  dx  ; Set GS to the current value (dx)
		mov ss,  cx  ; Set SS back to the current value (cx)
		mov bx,  gs  ; Move the value in gs into bx
		add eax, ebx ; Add the value in ebx to eax
		dec esi      ; Decrement the loop counter
	.until Zero?   ; Repeat until esi becomes 0
	mov    gs,  cx ; Set GS back to the current value (cx)
	div    ecx     ; Divide eax by ecx, storing the quotient in eax and the remainder in edx
	bt     eax, 3  ; Check if the third bit of eax is set
	ret            ; Return to the caller
TrCMGs    endp

; +
; Zero if !TF.
;
; This function checks if the Trace Flag (TF) is set in the current execution mode.
; It does this by calling the `NtYieldExecution` service, which is a special service that
; allows the operating system to yield control to another thread. In the context of this code,
; `NtYieldExecution` is used as a mechanism to check the TF flag indirectly.
;
; Returns:
;    0 if the TF flag is not set
;    Non-zero value if the TF flag is set
;
Tr86Sexit proc
	%ENVP Eax			; Get the pointer to the global environment structure (ENV)
	mov  eax, [eax].Yield   ; Get the address of the NtYieldExecution function from ENV
	Call NtGateSid          ; Call the `NtYieldExecution` function using the NT gate
	not  ecx                ; Invert the value of ecx (which contains the EFLAGS register)
	lea  eax, [ecx][esp][3] ; Calculate the address of the TF flag in the EFLAGS register
	ret                     ; Return to the caller
Tr86Sexit         endp

;
; EFLAGS -> R11 -> SYSRET
; 
; This function retrieves the value of the EFLAGS register from the R11 register 
; and then immediately performs a SYSRET instruction to return control to the caller.
;
; This function is specifically designed to be called using a `CALL64` instruction,
; which pushes the return address onto the stack and then performs a far jump to the
; target function. The function then retrieves the EFLAGS value from R11 and returns
; it to the caller by executing a `SYSRET` instruction.
;
; Note:
;   This function assumes that the caller has already set up the necessary registers
;   for the SYSRET instruction (e.g., RSP and RIP).
;
Tr64GetSexitFlags proc C
	DB 4CH, 89H, 0F8H	; mov rax,r11   - Move the value of R11 into RAX
	%RETFN			; SYSRET        - Execute the SYSRET instruction
Tr64GetSexitFlags endp

;
; Carry(TRUE) if TF.
;
; This function checks the Trace Flag (TF) in Compatibility Mode (CM) and sets the Carry Flag (CF) accordingly.
;
; In Compatibility Mode, the `NtYieldExecution` service is used to indirectly check the TF flag.
; The service is called through the NT gate, and the returned value in ECX contains the EFLAGS register.
; The function then extracts the TF flag bit (bit 8) and sets the Carry Flag (CF) if it is set.
;
; Returns:
;    CF = 1 (TRUE) if TF is set in Compatibility Mode.
;    CF = 0 (FALSE) if TF is not set in Compatibility Mode.
;
TrCMSexitFlags    proc
	%ENVP Eax			; Get the pointer to the global environment structure (ENV)
	mov   eax, [eax].Yield ; Get the address of the NtYieldExecution function from ENV
	Call  NtGateSid        ; Call the `NtYieldExecution` function using the NT gate
	%CALL64 Tr64GetSexitFlags	; Call the `Tr64GetSexitFlags` function to get EFLAGS from R11
	bt    eax, 8           ; Check if the TF flag bit (bit 8) is set in EFLAGS
	setc  al               ; Set the Carry Flag (CF) to 1 (TRUE) if TF is set, otherwise 0 (FALSE)
	movzx eax, al          ; Move the Carry Flag (CF) value into EAX
	ret                    ; Return to the caller
TrCMSexitFlags endp